/**
 * @author Radoslav Uhrin 2017
 * @email   
 * @website 
 * @link    
 * @version 
 * @ide    
 * @license 
 * @brief   Coding library for segment libraries
 * 
 * all definition in the defines.h
 *
 * Define segment type:
 * 
 * #define RU_CODING_FOR_7_SEGMENT	1
 * #define RU_CODING_FOR_14_SEGMENT	1
 *
 *	
 * Segment definiton:
 * ********************** 7 segment *******************
 *                        a
 *			*****
 *                   f *     * b
 *                     *  g  *
 *			*****
 *                   e *     * c
 *                     *     *
 *			*****
 *                        d
 *
 * deufalt position in coding string:
 *  7 6 5 4 3 2 1 0
 *  0 G F E D C B A
 * deufalt position (is possible change in to the defines.h):
 *
 * #define RU_7SEG_DISP_SEG_A	0x00
 * #define RU_7SEG_DISP_SEG_B	0x01
 * #define RU_7SEG_DISP_SEG_C	0x02
 * #define RU_7SEG_DISP_SEG_D	0x03
 * #define RU_7SEG_DISP_SEG_E	0x04
 * #define RU_7SEG_DISP_SEG_F	0x05
 * #define RU_7SEG_DISP_SEG_G	0x06
 *
 *
 * ********************** 14 segment *******************
 *            
 *      	    *****a*****
 *		   * *   *   * *
 *		   f  h  i  j  b
 *		   *   * * *   *
 *		    *g1** **g2* 
 *		   *   * * *   *
 *		   e  m  l  k  c
 *		   * *   *   * *
 *	            *****d*****
 *		
 *
 * deufalt position in coding string:
 *  13 12 11 10 09 08 07 06 05 04 03 02 01 00
 *   M  L  K  J  I  H G2 G1  F  E  D  C  B  A
 * deufalt position (is possible change in to the defines.h):
 *
 * #define RU_14SEG_DISP_SEG_A	0x00
 * #define RU_14SEG_DISP_SEG_B	0x01
 * #define RU_14SEG_DISP_SEG_C	0x02
 * #define RU_14SEG_DISP_SEG_D	0x03
 * #define RU_14SEG_DISP_SEG_E	0x04
 * #define RU_14SEG_DISP_SEG_F	0x05
 * #define RU_14SEG_DISP_SEG_G1	0x06
 * #define RU_14SEG_DISP_SEG_G2	0x07
 * #define RU_14SEG_DISP_SEG_H	0x08
 * #define RU_14SEG_DISP_SEG_I	0x09
 * #define RU_14SEG_DISP_SEG_J	0x0A
 * #define RU_14SEG_DISP_SEG_K	0x0B
 * #define RU_14SEG_DISP_SEG_L	0x0C
 * #define RU_14SEG_DISP_SEG_M	0x0D
 *
 *
 */
#ifndef RU_SEG_DISP_H
#define RU_SEG_DISP_H 110

//#include "stm32f4xx.h"
#include "defines.h"
//#include "string.h"

#ifndef RU_CODING_FOR_7_SEGMENT
#ifndef RU_CODING_FOR_14_SEGMENT
#ifndef RU_CODING_FOR_16_SEGMENT

#define RU_CODING_FOR_7_SEGMENT

#endif
#endif
#endif

//Segment definiton
#ifdef RU_CODING_FOR_7_SEGMENT

#ifndef RU_7SEG_DISP_SEG_A
#define RU_7SEG_DISP_SEG_A	0x00
#endif

#ifndef RU_7SEG_DISP_SEG_B
#define RU_7SEG_DISP_SEG_B	0x01
#endif

#ifndef RU_7SEG_DISP_SEG_C
#define RU_7SEG_DISP_SEG_C	0x02
#endif

#ifndef RU_7SEG_DISP_SEG_D
#define RU_7SEG_DISP_SEG_D	0x03
#endif

#ifndef RU_7SEG_DISP_SEG_E
#define RU_7SEG_DISP_SEG_E	0x04
#endif

#ifndef RU_7SEG_DISP_SEG_F
#define RU_7SEG_DISP_SEG_F	0x05
#endif

#ifndef RU_7SEG_DISP_SEG_G
#define RU_7SEG_DISP_SEG_G	0x06
#endif
#endif

#ifdef RU_CODING_FOR_14_SEGMENT

#ifndef RU_14SEG_DISP_SEG_A
#define RU_14SEG_DISP_SEG_A		0x00
#endif

#ifndef RU_14SEG_DISP_SEG_B
#define RU_14SEG_DISP_SEG_B		0x01
#endif

#ifndef RU_14SEG_DISP_SEG_C
#define RU_14SEG_DISP_SEG_C		0x02
#endif

#ifndef RU_14SEG_DISP_SEG_D
#define RU_14SEG_DISP_SEG_D		0x03
#endif

#ifndef RU_14SEG_DISP_SEG_E
#define RU_14SEG_DISP_SEG_E		0x04
#endif

#ifndef RU_14SEG_DISP_SEG_F
#define RU_14SEG_DISP_SEG_F		0x05
#endif

#ifndef RU_14SEG_DISP_SEG_G1
#define RU_14SEG_DISP_SEG_G1		0x06
#endif

#ifndef RU_14SEG_DISP_SEG_G2
#define RU_14SEG_DISP_SEG_G2		0x07
#endif

#ifndef RU_14SEG_DISP_SEG_H
#define RU_14SEG_DISP_SEG_H		0x08
#endif

#ifndef RU_14SEG_DISP_SEG_I
#define RU_14SEG_DISP_SEG_I		0x09
#endif

#ifndef RU_14SEG_DISP_SEG_J
#define RU_14SEG_DISP_SEG_J		0x0A
#endif

#ifndef RU_14SEG_DISP_SEG_K
#define RU_14SEG_DISP_SEG_K		0x0B
#endif

#ifndef RU_14SEG_DISP_SEG_L
#define RU_14SEG_DISP_SEG_L		0x0C
#endif

#ifndef RU_14SEG_DISP_SEG_M
#define RU_14SEG_DISP_SEG_M		0x0D
#endif

#endif

/**
* Coding 7seg disp
*
* Parameters
* 	- char ch - entry char - one digit
*	- const unsigned short *font_data
*
* Returns
*	- unsigned short decoding bytes
*/
extern unsigned RU_seg_disp_coding(char ch, const unsigned short *font_data);

/**
* Font data array
*
*
*/
extern const unsigned short RU_segment_coding [];

#endif

