/**
 * |----------------------------------------------------------------------
 * | Copyright (C) Radoslav Uhrin, 2018
 * |
 * |----------------------------------------------------------------------
 */
#include "ru_stm32_PT6315.h"

  //void Delayms(uint16_t _ms);

 /**
 * Clear RAM
 *
 * No parameters
 *
 * No returns
 */
void RU_PT6315_clear_RAM(void);

/**
* Set data format RAM
*
* Parameters:
* RU_PT6315_data_setting data_setting
*
* No returns
*/
void RU_PT6315_set_data_format(RU_PT6315_data_setting _data_setting);

/**
* Send data for one digit
*
* Parameters:
* unsigned _segmet_combination
*
* No returns
*/

void RU_PT6315_send_data_one_digit(unsigned _segment_combination);

extern void RU_PT6315_init(RU_PT6315_display_mode_enum _display_mode)
{

    #ifdef MCU_TYPE_STM32F4xx
	// pin inits
	GPIO_InitTypeDef GPIO_InitStruct;
	/* Enable clock for ports */
	RCC_AHB1PeriphClockCmd(PT6315_DIN_RCC, ENABLE);
	RCC_AHB1PeriphClockCmd(PT6315_DOUT_RCC, ENABLE);
	RCC_AHB1PeriphClockCmd(PT6315_STB_RCC, ENABLE);
	RCC_AHB1PeriphClockCmd(PT6315_CLK_RCC, ENABLE);

	/*DIN*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_DIN_PIN;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_DIN_PORT, &GPIO_InitStruct);

	/*DOUT*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_DOUT_PIN;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_DOUT_PORT, &GPIO_InitStruct);

	/*CLK*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_CLK_PIN;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_CLK_PORT, &GPIO_InitStruct);

	/*STB*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_STB_PIN;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_STB_PORT, &GPIO_InitStruct);

	#endif



	#ifdef MCU_TYPE_STM32F10x

	// pin inits
	GPIO_InitTypeDef GPIO_InitStruct;
	/* Enable clock for ports */
	RCC_APB2PeriphClockCmd(PT6315_DIN_RCC, ENABLE);
	RCC_APB2PeriphClockCmd(PT6315_DOUT_RCC, ENABLE);
	RCC_APB2PeriphClockCmd(PT6315_STB_RCC, ENABLE);
	RCC_APB2PeriphClockCmd(PT6315_CLK_RCC, ENABLE);

	/*DIN*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStruct.GPIO_Pin = PT6315_DIN_PIN;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_DIN_PORT, &GPIO_InitStruct);

	/*DOUT*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_DOUT_PIN;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_DOUT_PORT, &GPIO_InitStruct);

	/*CLK*/
	/*DOUT*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_CLK_PIN;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_CLK_PORT, &GPIO_InitStruct);

	/*STB*/
	/* Set settings */
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStruct.GPIO_Pin = PT6315_STB_PIN;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
	/* Init GPIO */
	GPIO_Init(PT6315_STB_PORT, &GPIO_InitStruct);


	#endif

	PT6315_STB_HIGH;
	PT6315_CLK_HIGH;
	PT6315_DOUT_HIGH;

	Delayms(200); //init delay

	/*Clear display RAM*/
	RU_PT6315_clear_RAM();

	Delayms(1000);

	/*Set display mode*/
	PT6315_STB_LOW;
	//Delayms(1000); //?
	RU_PT6315_write_8bit_data(PT6315_COMMAND_1_PREFIX | _display_mode);
	PT6315_STB_HIGH;

	/*Off display*/
	RU_PT6315_display_control(RU_PT6315_display_OFF_key_scan, 5);


	/* Set CLK, STP, DOUT high */
	PT6315_STB_HIGH;
	PT6315_CLK_HIGH;
	PT6315_DOUT_HIGH;


}

extern void RU_PT6315_display_control(RU_PT6315_display_control_enum _display_mode, uint8_t _dimming_display)
{
	PT6315_STB_LOW;
	//Delayms(20); //?
	RU_PT6315_write_8bit_data(PT6315_COMMAND_4_PREFIX | _display_mode | _dimming_display);
	PT6315_STB_HIGH;
}

extern void RU_PT6315_set_LED_output(RU_PT6315_LED_control_struct _led_control)
{
	uint8_t led_configuration = _led_control.LED_00 | (_led_control.LED_01 << 1) | (_led_control.LED_02 << 2) | (_led_control.LED_03 << 3);

	//set data format
	RU_PT6315_data_setting data_setting;
	data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_LED_port;
	data_setting.data_rw = RU_PT6315_adress_mode_increment;
	data_setting.mode = RU_PT6315_mode_normal;

	PT6315_STB_LOW;
	//Delayms(20); //?

	RU_PT6315_set_data_format(data_setting);

	PT6315_STB_HIGH;

	//Delayms(100); //?

	/*setting led*/

	PT6315_STB_LOW;
	//Delayms(20); //?
	RU_PT6315_write_8bit_data(led_configuration);

	PT6315_STB_HIGH;
}

extern void RU_PT6315_read_key_state(RU_PT6315_key_control_struct* _key_control, int8_t _couple_key_count)
{
	uint8_t i;
	RU_PT6315_data_setting data_setting;
	data_setting.adress_mode = RU_PT6315_data_rw_set_read_key_data;
	data_setting.data_rw = RU_PT6315_adress_mode_increment;
	data_setting.mode = RU_PT6315_mode_normal;

	PT6315_STB_LOW;
	//Delayms(20); //?

	RU_PT6315_set_data_format(data_setting);

	//Delayms(100); //waiting time

	for (i = 0; i < _couple_key_count; i++)
	{
		PT6315_CLK_LOW;
		//Delayms(20); //?

		//matrix 00
		PT6315_CLK_HIGH;
		//Delayms(20); //?

		if PT6315_DIN_PORT_READ
		{
			_key_control->key_matrix_00[i] = RU_PT6315_key_turn_on;
		}
		else
		{
			_key_control->key_matrix_00[i] = RU_PT6315_key_turn_off;
		}

		PT6315_CLK_LOW;
		//Delayms(20); //?

		//matrix 01
		PT6315_CLK_HIGH;
		//Delayms(20); //?

		if PT6315_DIN_PORT_READ
		{
			_key_control->key_matrix_01[i] = RU_PT6315_key_turn_on;
		}
		else
		{
			_key_control->key_matrix_01[i] = RU_PT6315_key_turn_off;
		}

		PT6315_CLK_LOW;
		//Delayms(20); //?

	}

	PT6315_STB_HIGH;
}

extern void RU_PT6315_display_test_mode(RU_PT6315_mode_enum _mode)
{
	RU_PT6315_data_setting data_setting;
	data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
	data_setting.data_rw = RU_PT6315_adress_mode_increment;
	data_setting.mode = _mode;

	PT6315_STB_LOW;
	//Delayms(20); //?

	RU_PT6315_set_data_format(data_setting);

	PT6315_STB_HIGH;
}

extern void RU_PT6315_set_digit(uint32_t _segment_combination, uint8_t _digit_number)
{
	//set data format
	RU_PT6315_data_setting data_setting;
	data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
	data_setting.data_rw = RU_PT6315_adress_mode_increment;
	data_setting.mode = RU_PT6315_mode_normal;

	PT6315_STB_LOW;
	//Delayms(20); //?

	RU_PT6315_set_data_format(data_setting);

	PT6315_STB_HIGH;

	//Delayms(1000); //?

	//send start command

	PT6315_STB_LOW;
	//Delayms(20); //?
	uint8_t start_adress_command = PT6315_COMMAND_3_PREFIX | (_digit_number * 3);
	RU_PT6315_write_8bit_data(start_adress_command);

	//Delayms(1000); //?

	//send data
	RU_PT6315_send_data_one_digit(_segment_combination);

	PT6315_STB_HIGH;
	//Delayms(20); //?
	PT6315_DOUT_HIGH;
}

extern void RU_PT6315_set_digits(uint32_t* _segment_combination, uint8_t _start_digit_number, uint8_t _digit_count)
{
    //chceck if is not digits place position over possibilities
    if ((_start_digit_number + _digit_count) > 11) _digit_count = 11 - _start_digit_number;

    //set data format
    RU_PT6315_data_setting data_setting;
    data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
    data_setting.data_rw = RU_PT6315_adress_mode_increment;
    data_setting.mode = RU_PT6315_mode_normal;

    PT6315_STB_LOW;
    //Delayms(20); //?

    RU_PT6315_set_data_format(data_setting);

    PT6315_STB_HIGH;
    //Delayms(100); //?

    //send start command

    PT6315_STB_LOW;
    //Delayms(20); //?

    uint8_t start_adress_command = PT6315_COMMAND_3_PREFIX | (_start_digit_number * 3);
    RU_PT6315_write_8bit_data(start_adress_command);

    //Delayms(100); //?

    uint8_t digit_position;

    //digit loop

    for (digit_position = 0; digit_position < _digit_count; digit_position++)
    {
        //send data
        RU_PT6315_send_data_one_digit(_segment_combination[digit_position]);
    }

    PT6315_STB_HIGH;
    //Delayms(20); //?
    PT6315_DOUT_HIGH;
}

extern void RU_PT6315_put_chars(char* str, uint8_t _start_digit_number)
{
  //set data format
  RU_PT6315_data_setting data_setting;
  data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
  data_setting.data_rw = RU_PT6315_adress_mode_increment;
  data_setting.mode = RU_PT6315_mode_normal;

  PT6315_STB_LOW;
  RU_PT6315_set_data_format(data_setting);
  PT6315_STB_HIGH;


  //send start command
  PT6315_STB_LOW;
  uint8_t start_adress_command = PT6315_COMMAND_3_PREFIX | (_start_digit_number * 3);
  RU_PT6315_write_8bit_data(start_adress_command);

  uint8_t digit_position= 0;
  unsigned _segment_combination;

  // string loop
  while (*str)
  {
      _segment_combination=RU_seg_disp_coding(*str, RU_segment_coding);

      //send data (one digit)
      RU_PT6315_send_data_one_digit( _segment_combination);

    /* Increase string pointer */
    str++;

    /* Increase digit_position */
    digit_position++;

    /* Check if is not in the end of the digits */
    if ((digit_position+_start_digit_number)>11) break;

  }

  PT6315_STB_HIGH;
  PT6315_DOUT_HIGH;
}

extern void RU_PT6315_put_chars_with_signs(char* str, uint8_t _start_digit_number, RU_PT6315_sign_control_struct* _signs, uint8_t _sign_count)
{
  //set data format
  RU_PT6315_data_setting data_setting;
  data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
  data_setting.data_rw = RU_PT6315_adress_mode_increment;
  data_setting.mode = RU_PT6315_mode_normal;

  PT6315_STB_LOW;
  RU_PT6315_set_data_format(data_setting);
  PT6315_STB_HIGH;


  //send start command
  PT6315_STB_LOW;
  uint8_t start_adress_command = PT6315_COMMAND_3_PREFIX | (_start_digit_number * 3);
  RU_PT6315_write_8bit_data(start_adress_command);

  uint8_t digit_position= _start_digit_number;
  uint8_t i, sign, seg_flag;
  unsigned _segment_combination;

  // string loop
  while (*str)
  {
      _segment_combination=RU_seg_disp_coding(*str, RU_segment_coding);

      //look for sign in sanded string

      for (sign = 0; sign < _sign_count; ++sign)
	{
	if (_signs[sign].digit == digit_position)
	  {
	    if (_signs[sign].sign_control==RU_PT6315_sign_on )
	      {
		_segment_combination|=_signs[sign].segment_combination;
	      }
	  }
	}

      //send data (one digit)
      RU_PT6315_send_data_one_digit( _segment_combination);

    /* Increase string pointer */
    str++;

    /* Increase digit_position */
    digit_position++;

    /* Check if is not in the end of the digits */
    if ((digit_position)>11) break;
  }

  //check sign which was not in the string:
  //Before text:

  for ( i = 0; i < _start_digit_number; ++i)
  {
      _segment_combination=0;
      seg_flag=0;

      for (sign = 0; sign < _sign_count; ++sign)
	{
	 if (_signs[sign].digit == i)
      	  {
	    seg_flag=1;
      	    if (_signs[sign].sign_control==RU_PT6315_sign_on ) _segment_combination|=_signs[sign].segment_combination;
      	  }
	}
      if (seg_flag)
	{
	  PT6315_STB_HIGH;
	  PT6315_STB_LOW;
	  start_adress_command = PT6315_COMMAND_3_PREFIX | (i * 3);
	  RU_PT6315_write_8bit_data(start_adress_command);
	  RU_PT6315_send_data_one_digit( _segment_combination);
	}
  }

  //After text:
  for (i = digit_position+2 ; i < 12; ++i)
    {
	_segment_combination=0;
	seg_flag=0;

        for (sign = 0; sign < _sign_count; ++sign)
          {
  	 if (_signs[sign].digit == i)
  	   {
             seg_flag=1;
             if (_signs[sign].sign_control==RU_PT6315_sign_on ) _segment_combination|=_signs[sign].segment_combination;
  	   }
          }
        if (seg_flag)
          {
            PT6315_STB_HIGH;
            PT6315_STB_LOW;
            start_adress_command = PT6315_COMMAND_3_PREFIX | (i * 3);
            RU_PT6315_write_8bit_data(start_adress_command);
            RU_PT6315_send_data_one_digit( _segment_combination);
          }
    }

  PT6315_STB_HIGH;
  PT6315_DOUT_HIGH;
}

void RU_PT6315_send_data_one_digit(unsigned _segment_combination)
{

	for (uint8_t i = 0; i < 0x18; i++)
	{
	PT6315_CLK_LOW;

	if ((_segment_combination >> i) & 0x01)
	  {
	    PT6315_DOUT_HIGH;
	  }
	else
	  {
	    PT6315_DOUT_LOW;
	  }

	PT6315_CLK_HIGH;
	}
}

extern void RU_PT6315_write_data(uint8_t* _data, uint8_t _count, uint8_t _start_adress)
{
	//set data format
	RU_PT6315_data_setting data_setting;
	data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
	data_setting.data_rw = RU_PT6315_adress_mode_increment;
	data_setting.mode = RU_PT6315_mode_normal;

	PT6315_STB_LOW;
	//Delayms(20); //?

	RU_PT6315_set_data_format(data_setting);

	PT6315_STB_HIGH;
	//Delayms(100); //?

				 //send start command

	PT6315_STB_LOW;
	//Delayms(20); //?
	uint8_t start_adress_command = PT6315_COMMAND_3_PREFIX | _start_adress;
	RU_PT6315_write_8bit_data(start_adress_command);

	//Delayms(100); //?

	uint8_t i;

	for (i = 0; i < _count; i++)
	{
		RU_PT6315_write_8bit_data(_data[i]);
	}

	PT6315_STB_HIGH;
	//Delayms(20); //?
	PT6315_DOUT_HIGH;

}


void RU_PT6315_clear_RAM(void)
{
	//set data format


	RU_PT6315_data_setting data_setting;
	data_setting.adress_mode = RU_PT6315_data_rw_set_write_data_to_display;
	data_setting.data_rw = RU_PT6315_adress_mode_increment;
	data_setting.mode = RU_PT6315_mode_normal;

	PT6315_STB_LOW;
	//Delayms(20); //?

	RU_PT6315_set_data_format(data_setting);

	PT6315_STB_HIGH;

	//Delayms(10);

	//set start adress
	uint8_t start_adress_command = PT6315_COMMAND_3_PREFIX | 0; // 0 - start adress

	//send start command
	PT6315_STB_LOW;
	//Delayms(20); //?
	RU_PT6315_write_8bit_data(start_adress_command);

	//Delayms(10);

	//clear RAM
	//0x24*8 bits = 0x120
	PT6315_DOUT_LOW;

	uint16_t i;
	for (i = 0; i < 0x120; i++)
	{
		PT6315_CLK_LOW;
		Delayms(1); //?
		PT6315_CLK_HIGH;
		Delayms(1); //?
	}

	PT6315_STB_HIGH;
	//Delayms(20); //?
	PT6315_DOUT_HIGH;
}

void RU_PT6315_set_data_format(RU_PT6315_data_setting _data_setting)
{
	RU_PT6315_write_8bit_data(PT6315_COMMAND_2_PREFIX |
		_data_setting.mode |
		_data_setting.adress_mode |
		_data_setting.data_rw);

}

void RU_PT6315_write_8bit_data(uint8_t _data)
{
	uint8_t i;

	for (i = 0; i < 8; i++)
	{
		PT6315_CLK_LOW;

		if ((_data >> i) & 0x01)
		{
			PT6315_DOUT_HIGH;
		}
		else
		{
			PT6315_DOUT_LOW;
		}

		//Delayms(20); //?

		PT6315_CLK_HIGH;

		//Delayms(20); //?
	}
	PT6315_DOUT_HIGH;
}
/*
void Delayms(uint16_t _ms)
{
  uint16_t a;
  for ( a = 0; a < _ms; ++a) {

  }
}

*/
