/**
 * PT635 Library for STM32F100, STM32F4xx,  - VFD Driver
 *
 * Default MCU type (Default type are MCU_TYPE_STM32F4xx):
 *
 *  	#define MCU_TYPE_STM32F4xx
 *  	#define MCU_TYPE_STM32F10x
 *
 *
 * Connection:
 *
 *  STM             PT6315
 *
 *  DIN     <--     DOUT
 *  DOUT    -->     DIN
 *  CLK     -->     CLK
 *  STB     -->     STB
 *
 * Default pinout (MCU_TYPE_STM32F4xx)
 *
 *
 *  	DIN pin - Data input from PT6315
 *
 *  	#define PT6315_DIN_RCC				RCC_AHB1Periph_GPIOD
 *  	#define PT6315_DIN_PORT				GPIOD
 *  	#define PT6315_DIN_PIN				GPIO_Pin_0
 *
 *  	DOUT pin - Data output to PT6315
 *
 *  	#define PT6315_DOUT_RCC				RCC_AHB1Periph_GPIOD
 *  	#define PT6315_DOUT_PORT            GPIOD
 *  	#define PT6315_DOUT_PIN				GPIO_Pin_1
 *
 *  	CLK pin - Clock output to PT6315
 *
 *  	#define PT6315_CLK_RCC				RCC_AHB1Periph_GPIOD
 *  	#define PT6315_CLK_PORT             GPIOD
 *  	#define PT6315_CLK_PIN				GPIO_Pin_2
 *
 *  	STB pin - strobe output to PT6315
 *
 *  	#define PT6315_STB_RCC				RCC_AHB1Periph_GPIOD
 *  	#define PT6315_STB_PORT             GPIOD
 *  	#define PT6315_STB_PIN				GPIO_Pin_3
 */

#ifndef RU_PT6315
#define RU_PT6315 100
/**
 * Library dependencies
 * - SRU32F4xx
 * - SRU32F4xx RCC
 * - SRU32F4xx GPIO
 * - SRU32F4xx SPI
 * - RU SPI
 * - defines.h
 */
/**
 * Includes
 */

#include "defines.h"


 // MCU type defines
#ifndef MCU_TYPE_STM32F4xx

#ifndef MCU_TYPE_STM32F10x

#define MCU_TYPE_STM32F4xx

#endif

#endif



#ifdef MCU_TYPE_STM32F4xx

#include "stm32f4xx_gpio.h"
#include "tm_stm32f4_delay.h"

#endif

#ifdef MCU_TYPE_STM32F10x

#include "stm32f10x.h"
#include "stm32f10x_gpio.h"
#include "tm_stm32f10x_delay.h"

#endif

// coding function

#include "ru_seg_disp_coding.h"

// specify pins

#ifdef MCU_TYPE_STM32F4xx

// DIN
#ifndef PT6315_DIN_RCC
#define PT6315_DIN_RCC				RCC_AHB1Periph_GPIOD
#endif

#ifndef PT6315_DIN_PORT
#define PT6315_DIN_PORT				GPIOD
#endif

#ifndef PT6315_DIN_PIN
#define PT6315_DIN_PIN				GPIO_Pin_0
#endif

// DOUT
#ifndef PT6315_DOUT_RCC
#define PT6315_DOUT_RCC				RCC_AHB1Periph_GPIOD
#endif

#ifndef PT6315_DOUT_PORT
#define PT6315_DOUT_PORT            		GPIOD
#endif

#ifndef PT6315_DOUT_PIN
#define PT6315_DOUT_PIN				GPIO_Pin_1
#endif

// CLK
#ifndef PT6315_CLK_RCC
#define PT6315_CLK_RCC				RCC_AHB1Periph_GPIOD
#endif

#ifndef PT6315_CLK_PORT
#define PT6315_CLK_PORT             		GPIOD
#endif

#ifndef PT6315_CLK_PIN
#define PT6315_CLK_PIN				GPIO_Pin_2
#endif

// STB
#ifndef PT6315_STB_RCC
#define PT6315_STB_RCC				RCC_AHB1Periph_GPIOD
#endif

#ifndef PT6315_STB_PORT
#define PT6315_STB_PORT             		GPIOD
#endif

#ifndef PT6315_STB_PIN
#define PT6315_STB_PIN				GPIO_Pin_3
#endif

/* Pin macros */
#define PT6315_DOUT_LOW             (PT6315_DOUT_PORT->BSRRH = PT6315_DOUT_PIN)
#define PT6315_DOUT_HIGH            (PT6315_DOUT_PORT->BSRRL = PT6315_DOUT_PIN)

#define PT6315_CLK_LOW              (PT6315_CLK_PORT->BSRRH = PT6315_CLK_PIN)
#define PT6315_CLK_HIGH             (PT6315_CLK_PORT->BSRRL = PT6315_CLK_PIN)

#define PT6315_STB_LOW              (PT6315_STB_PORT->BSRRH = PT6315_STB_PIN)
#define PT6315_STB_HIGH             (PT6315_STB_PORT->BSRRL = PT6315_STB_PIN)

#define PT6315_DIN_PORT_READ        (PT6315_DIN_PORT->IDR & (PT6315_DIN_PIN))

#endif

#ifdef MCU_TYPE_STM32F10x
// DIN
#ifndef PT6315_DIN_RCC
#define PT6315_DIN_RCC				RCC_APB2Periph_GPIOA
#endif

#ifndef PT6315_DIN_PORT
#define PT6315_DIN_PORT				GPIOA
#endif

#ifndef PT6315_DIN_PIN
#define PT6315_DIN_PIN				GPIO_Pin_0
#endif

// DOUT
#ifndef PT6315_DOUT_RCC
#define PT6315_DOUT_RCC				RCC_APB2Periph_GPIOA
#endif

#ifndef PT6315_DOUT_PORT
#define PT6315_DOUT_PORT			GPIOA
#endif

#ifndef PT6315_DOUT_PIN
#define PT6315_DOUT_PIN				GPIO_Pin_1
#endif

// CLK
#ifndef PT6315_CLK_RCC
#define PT6315_CLK_RCC				RCC_APB2Periph_GPIOA
#endif

#ifndef PT6315_CLK_PORT
#define PT6315_CLK_PORT				GPIOA
#endif

#ifndef PT6315_CLK_PIN
#define PT6315_CLK_PIN				GPIO_Pin_2
#endif

// STB
#ifndef PT6315_STB_RCC
#define PT6315_STB_RCC				RCC_APB2Periph_GPIOA
#endif

#ifndef PT6315_STB_PORT
#define PT6315_STB_PORT				GPIOA
#endif

#ifndef PT6315_STB_PIN
#define PT6315_STB_PIN				GPIO_Pin_3
#endif

/* Pin macros*/

#define PT6315_DOUT_LOW				PT6315_DOUT_PORT->BRR = (PT6315_DOUT_PIN)
#define PT6315_DOUT_HIGH			PT6315_DOUT_PORT->BSRR = (PT6315_DOUT_PIN)

#define PT6315_CLK_LOW				PT6315_CLK_PORT->BRR = (PT6315_CLK_PIN)
#define PT6315_CLK_HIGH				PT6315_CLK_PORT->BSRR = (PT6315_CLK_PIN)
#define PT6315_CLK_TOGGLE			PT6315_CLK_PORT->ODR ^= (PT6315_CLK_PIN);

#define PT6315_STB_LOW				PT6315_STB_PORT->BRR = (PT6315_STB_PIN)
#define PT6315_STB_HIGH				PT6315_STB_PORT->BSRR = (PT6315_STB_PIN)


#define PT6315_DIN_PORT_READ			(PT6315_DIN_PORT->IDR & (PT6315_DIN_PIN))
#endif


//defines:
#define PT6315_COMMAND_1_PREFIX		0x00
#define PT6315_COMMAND_2_PREFIX		0x40
#define PT6315_COMMAND_3_PREFIX		0xC0
#define PT6315_COMMAND_4_PREFIX		0x80


/*COMMAND 1 - display mode*/
/**
 *
 * Enumeration for displey mode
 *
 * Default  - RU_PT6315_display_mode_12_dig_16_seg
 */
typedef enum RU_PT6315_display_mode_enum
{
    RU_PT6315_display_mode_04_dig_24_seg            = 0x00,
    RU_PT6315_display_mode_05_dig_23_seg            = 0x01,
    RU_PT6315_display_mode_06_dig_22_seg            = 0x02,
    RU_PT6315_display_mode_07_dig_21_seg            = 0x03,
    RU_PT6315_display_mode_08_dig_20_seg            = 0x04,
    RU_PT6315_display_mode_09_dig_19_seg            = 0x05,
    RU_PT6315_display_mode_10_dig_18_seg            = 0x06,
    RU_PT6315_display_mode_11_dig_17_seg            = 0x07,
    RU_PT6315_display_mode_12_dig_16_seg            = 0x08

} RU_PT6315_display_mode_enum;

/*COMMAND 2 - data setting commands*/

/**
 * Enumeration for write & Read mode setting
 *
 * Default  - RU_PT6315_data_rw_set_write_data_to_disp
 */
typedef enum RU_PT6315_data_rw_setting_enum
{
    RU_PT6315_data_rw_set_write_data_to_display		= 0x00,
    RU_PT6315_data_rw_set_write_data_to_LED_port	= 0x01,
    RU_PT6315_data_rw_set_read_key_data			= 0x02

} RU_PT6315_data_rw_setting_enum;

/**
 * Enumeration for adress mode setting.
 *
 * Default  - RU_PT6315_adress_mode_increment
 */
typedef enum RU_PT6315_adress_mode_enum
{
    RU_PT6315_adress_mode_increment                 = 0x00,
    RU_PT6315_adress_mode_fixes                     = 0x04

} RU_PT6315_adress_mode_enum;

/**
 * Enumeration for mode setting.
 *
 * Default  - RU_PT6315_mode_normal
 */
typedef enum RU_PT6315_mode_enum
{
    RU_PT6315_mode_normal                           = 0x00,
    RU_PT6315_mode_test                             = 0x08

} RU_PT6315_mode_enum;

/**
* Structurn for data setting.
*
*/
typedef struct RU_PT6315_data_setting
{
	RU_PT6315_data_rw_setting_enum			data_rw;
	RU_PT6315_adress_mode_enum			adress_mode;
	RU_PT6315_mode_enum				mode;

} RU_PT6315_data_setting;



/**
* Enumeration for display control.
*
* Default  - RU_PT6315_mode_normal
*/
typedef enum RU_PT6315_display_control_enum
{
	RU_PT6315_display_ON				= 0x08,
	RU_PT6315_display_OFF_key_scan			= 0x00

} RU_PT6315_display_control_enum;


/**
* Enumeration for LED control.
*
* Default  - RU_PT6315_LED_turn_on
*/
typedef enum RU_PT6315_LED_status_enum
{
	RU_PT6315_LED_turn_on				= 0x01,
	RU_PT6315_LED_turn_off				= 0x00,

} RU_PT6315_LED_status_enum;

/**
* Led control structure
*
*/

typedef struct RU_PT6315_LED_control_struct
{
	RU_PT6315_LED_status_enum			LED_00;
	RU_PT6315_LED_status_enum			LED_01;
	RU_PT6315_LED_status_enum			LED_02;
	RU_PT6315_LED_status_enum			LED_03;

} RU_PT6315_LED_control_struct;

/**
* Enumeration for Key control (read actual state).
*
*/
typedef enum RU_PT6315_key_status
{
	RU_PT6315_key_turn_on				= 0x01,
	RU_PT6315_key_turn_off				= 0x00,

} RU_PT6315_key_status;

/**
* key control structure
*
*/
typedef struct RU_PT6315_key_control_struct
{
	RU_PT6315_key_status				key_matrix_00[16];
	RU_PT6315_key_status				key_matrix_01[16];

} RU_PT6315_key_control_struct;

/**
* sign control enum
*
*/
typedef enum RU_PT6315_sign_control_enum
{
	RU_PT6315_sign_on				= 0x01,
	RU_PT6315_sign_off				= 0x00,

} RU_PT6315_sign_control_enum;

/**
**
* sign control structure
*
*/
typedef struct RU_PT6315_sign_control_struct
{
	uint8_t						digit;
	uint8_t						segment_combination;
	RU_PT6315_sign_control_enum			sign_control;

} RU_PT6315_sign_control_struct;


/**
 * Basic init
 *
 * Parameters
 * 	- RU_PT6315_display_mode_enum
 *
 * No returns
 */
extern void RU_PT6315_init(RU_PT6315_display_mode_enum _display_mode);

/**
* Display control
*
* Parameters
* 	- RU_PT6315_display_control_enum
*	- uint8_t dimming_display - 0-7
*
* No returns
*/
extern void RU_PT6315_display_control(RU_PT6315_display_control_enum _display_mode, uint8_t _dimming_display);

/**
* Set LED output
*
* Parameters
* 	RU_PT6315_LED_control_struct led_control
*
* No returns
*/
extern void RU_PT6315_set_LED_output(RU_PT6315_LED_control_struct _led_control);

/**
* Read key state
*
* Parameters
* pointer at RU_PT6315_key_control_struct - key_control
* int8_t  couple key count - (key0 - matrix0, key0 - matrix1) -> couple
*
* No returns
*/
extern void RU_PT6315_read_key_state(RU_PT6315_key_control_struct* _key_control, int8_t _couple_key_count);

/**
* Activate test mode
*
* Parameters
* pointer at RU_PT6315_key_control_struct - key_control
*
* No returns
*/
extern void RU_PT6315_display_test_mode(RU_PT6315_mode_enum _mode);

/**
* Set one digit
*
* Parameters
* 	- uint32_t segment_combination - max 24 bit (3x8bits)
*	- uint8_t digit_number - 0-11
*
* No returns
*/
extern void RU_PT6315_set_digit(uint32_t _segment_combination, uint8_t _digit_number);

/**
* Set digit
*
* Parameters
*    - uint32_t* point at array with segment_combination - max 24 bit (3x8bits) for every segment
*    - uint8_t _start_digit_number - 0-11
*    - uint8_t _digit_count - 1-11 (_start_digit_number + _digit_count <= 11 !)
*
* No returns
*/
extern void RU_PT6315_set_digits(uint32_t* _segment_combination, uint8_t _start_digit_number, uint8_t _digit_count);

/**
* Put chars
*
* Parameters
*    - uint32_t* point at string array (Max 11 strings contains start digit! Only first 11 strings, with start digit,  will be displayed)
*    - uint8_t _start_digit_number - 0-11
*
* No returns
*/
extern void RU_PT6315_put_chars(char* str, uint8_t _start_digit_number);


/**
* Put chars with signs
*
* Parameters
*    - uint32_t* point at string array (Max 11 strings! Only first 11 strings will be displayed)
*    - uint8_t _start_digit_number - 0-11
*    - uint8_t _sign_count (Number of the predefined sign - array size)
*
* No returns
*/
extern void RU_PT6315_put_chars_with_signs(char* str, uint8_t _start_digit_number, RU_PT6315_sign_control_struct* _signs, uint8_t _sign_count);




/**
* Write command
*
* Parameters
* 	- uint8_t comand
*
* No returns
*/
extern void RU_PT6315_write_8bit_data(uint8_t _data);

/**
* Write data
*
* Parameters
* 	- uint8_t* data - pointer at data array
* 	- uint8_t* count
* 	- uint8_t start_adress
*
* No returns
*/
extern void RU_PT6315_write_data(uint8_t* _data, uint8_t _count, uint8_t _start_adress);

#endif

